@@ -434,7 +434,7 @@ class Book:
         # text["text"] = text["text"].str.replace(r":", "", regex=True)
         # text["text"] = text["text"].str.replace(r"é", "e", regex=True)
 
-        text.text = text.text.str.strip()
+        text['text'] = text['text'].str.strip()
         text = text[text.text != ""]
         text = text.reset_index(drop=True)
 
@@ -680,27 +680,40 @@ class Book:
         # Create lines bases on commas and '-'
         # change other strange symbols/characters
         text["text"] = text["text"].str.replace(r",,", ".,", regex=True)
-        text["text"] = text["text"].str.replace(r"-—", "— ", regex=True)
-        text["text"] = text["text"].str.replace(r"-", "— ", regex=True)
-        text["text"] = text["text"].str.replace(r"—", "— ", regex=True)
+        text["text"] = text["text"].str.replace(r"-—", "—", regex=True)
+        text["text"] = text["text"].str.replace(r"-", "—", regex=True)
+        text["text"] = text["text"].str.replace(r"—", "—", regex=True)
         text["text"] = text["text"].str.replace(r"\.—", "—", regex=True)
         text["text"] = text["text"].str.replace(r"—[ ]{0,1}—", "—", regex=True)
         text["text"] = text["text"].str.replace(r"— ,", "—", regex=True)
         text["text"] = text["text"].str.replace(r", —", "—", regex=True)
 
+        text['text'] = text['text'].str.strip()
+
         length = text.shape[0]
         text["end"] = text["text"].astype(str).str[-1]
 
-        for i in range(length-1, -1, -1):
-            try:
-                if (text.at[i, "end"] == "," or text.at[i, "end"] == "-" or text.at[i, "end"] == "—" or re.search("^—[ ]{0,1}[0-9]{3,}$", text.at[i+1, "text"])) and text.at[i, "page"] == text.at[i+1, "page"]:
-                    text.at[i, "text"] = text.at[i, "text"] + " " + text.at[i+1, "text"]
-                    text.at[i+1, "text"] = ""
-            except:
-                pass
+        for i in range(length-2, -1, -1):
+            text.at[i, 'DUMMY'] = 2
+            if (text.at[i, "end"] == ","):
+                text.at[i, 'DUMMY'] = 1
+            if text.at[i, "end"] == "-":
+                text.at[i, 'DUMMY'] = 1
+            if text.at[i, "end"] == "—" and re.search("^[0-9]{3,}$", str(text.at[i+1, "text"])):
+                text.at[i, 'DUMMY'] = 1
+            if re.search("^—[ ]{0,1}[0-9]{3,}$", str(text.at[i+1, "text"])):
+                text.at[i, 'DUMMY'] = 1
+            if text.at[i, "page"] != text.at[i+1, "page"]:
+                text.at[i, 'DUMMY'] = 0
+
+        for i in range(length-2, -1, -1):
+            if (text.at[i, "DUMMY"] == 1):
+                text.at[i, "text"] = str(text.at[i, "text"]) + " " + str(text.at[i+1, "text"])
+                text.at[i+1, "text"] = ""
 
         text = text[text.text != ""]
         text = text.drop(['end'], axis=1)
+        text = text.drop(['DUMMY'], axis=1)
         text = text.reset_index(drop=True)
 
         return text
@@ -1008,20 +1021,27 @@ class Book:
 
         return text
 
-    # ===========`
+    # ===========
 
     def RegexTaxeringskalender(self, df):
         """Run predefined regex on dataframe."""
         global csv
 
-        csv = df
+        csv = df.copy()
+        csv["text"] = csv["text"].str.replace(r"([0-9]{3,}) [a-zæøå]$", "\\1", regex=True)
 
-        extract = csv['text'].str.extract(r"(^.*?[^0-9])([0-9]*)[—\- ]{1,3}([0-9]{1,})$")
+        extract = csv['text'].str.extract(r"^(.*?) ([a-xæøåäö., —]{3,})[ ]*([A-ZÆØÅÄÖ]{0,1}[^0-9]*?)([0-9]*)[—\- ]{1,3}([0-9]{1,})$")
+        extract = extract.rename(columns={"0": "name",
+                                          "1": "occupation",
+                                          "2": "residence",
+                                          "3": "income",
+                                          "4": "wealth"})
         extract = extract.join(csv, how='outer')
 
         del csv
         extract = extract.drop(['Unnamed: 0', 'level', 'page_num', 'par_num',
-                               'line_num', 'word_num', 'width', 'conf'],
+                                'line_num', 'word_num', 'width', 'conf',
+                                'text_part'],
                                axis=1, errors='ignore')
 
         return extract
